/**
 * Copyright (c) 2013 Stefan Marr, stefan.marr@vub.ac.be
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package som.interpreter;

import java.math.BigInteger;

import com.oracle.truffle.api.CompilerAsserts;
import com.oracle.truffle.api.dsl.TypeSystem;

import som.vmobjects.SAbstractObject;


@TypeSystem({boolean.class,
    long.class,
    BigInteger.class,
    String.class,
    double.class,
    SAbstractObject.class,
    Object[].class}) // Object[] is only for argument passing
public class Types {

  /** Return String representation of obj to be used in debugger. */
  public static String toDebuggerString(final Object obj) {
    if (obj instanceof Boolean) {
      if ((boolean) obj) {
        return "true";
      } else {
        return "false";
      }
    }
    if (obj instanceof String) {
      return (String) obj;
    }
    return obj.toString();

  }

  public static int getNumberOfNamedSlots(final Object obj) {
    CompilerAsserts.neverPartOfCompilation();

    // think, only SObject has fields
    return 0;
  }

  public static int getNumberOfIndexedSlots(final Object obj) {
    CompilerAsserts.neverPartOfCompilation();

    return 0;

  }
}
